@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -36,7 +36,6 @@
 #include "PeerInitiateConnectionCommand.h"
 #include "message.h"
 #include "DownloadEngine.h"
-#include "BtContext.h"
 #include "PeerStorage.h"
 #include "PieceStorage.h"
 #include "BtRuntime.h"
@@ -48,19 +47,21 @@
 #include "SocketCore.h"
 #include "BtAnnounce.h"
 #include "RequestGroup.h"
+#include "DownloadContext.h"
+#include "bittorrent_helper.h"
+#include "wallclock.h"
+#include "util.h"
 
 namespace aria2 {
 
 ActivePeerConnectionCommand::ActivePeerConnectionCommand
-(int cuid,
+(cuid_t cuid,
  RequestGroup* requestGroup,
  DownloadEngine* e,
- const BtContextHandle& btContext,
  time_t interval)
   :
   Command(cuid),
   _requestGroup(requestGroup),
-  _btContext(btContext),
   interval(interval),
   e(e),
   _numNewConnection(5)
@@ -77,44 +78,50 @@ bool ActivePeerConnectionCommand::execute() {
   if(_btRuntime->isHalt()) {
     return true;
   }
-  if(checkPoint.elapsed(interval)) {
-    checkPoint.reset();
+  if(checkPoint.difference(global::wallclock) >= interval) {
+    checkPoint = global::wallclock;
     TransferStat tstat = _requestGroup->calculateStat();
     const unsigned int maxDownloadLimit =
       _requestGroup->getMaxDownloadSpeedLimit();
     const unsigned int maxUploadLimit = _requestGroup->getMaxUploadSpeedLimit();
-    unsigned int thresholdSpeed =
-      _requestGroup->getOption()->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT);
+    unsigned int thresholdSpeed;
+    if(_requestGroup->getDownloadContext()->
+       getAttribute(bittorrent::BITTORRENT).containsKey(bittorrent::METADATA)) {
+      thresholdSpeed =
+        _requestGroup->getOption()->getAsInt(PREF_BT_REQUEST_PEER_SPEED_LIMIT);
+    } else {
+      thresholdSpeed = 0;
+    }
     if(maxDownloadLimit > 0) {
       thresholdSpeed = std::min(maxDownloadLimit, thresholdSpeed);
     }
     if(// for seeder state
        (_pieceStorage->downloadFinished() && _btRuntime->lessThanMaxPeers() &&
-	(maxUploadLimit == 0 || tstat.getUploadSpeed() < maxUploadLimit*0.8)) ||
+        (maxUploadLimit == 0 || tstat.getUploadSpeed() < maxUploadLimit*0.8)) ||
        // for leecher state
        (!_pieceStorage->downloadFinished() &&
-	(tstat.getDownloadSpeed() < thresholdSpeed ||
-	 _btRuntime->lessThanMinPeers()))) {
+        (tstat.getDownloadSpeed() < thresholdSpeed ||
+         _btRuntime->lessThanMinPeers()))) {
 
       unsigned int numConnection = 0;
       if(_pieceStorage->downloadFinished()) {
-	if(_btRuntime->getMaxPeers() > _btRuntime->getConnections()) {
-	  numConnection =
-	    std::min(_numNewConnection,
-		     _btRuntime->getMaxPeers()-_btRuntime->getConnections());
-	}
+        if(_btRuntime->getMaxPeers() > _btRuntime->getConnections()) {
+          numConnection =
+            std::min(_numNewConnection,
+                     _btRuntime->getMaxPeers()-_btRuntime->getConnections());
+        }
       } else {
-	numConnection = _numNewConnection;
+        numConnection = _numNewConnection;
       }
 
       for(unsigned int numAdd = numConnection;
-	  numAdd > 0 && _peerStorage->isPeerAvailable(); --numAdd) {
-	PeerHandle peer = _peerStorage->getUnusedPeer();
-	connectToPeer(peer);
+          numAdd > 0 && _peerStorage->isPeerAvailable(); --numAdd) {
+        SharedHandle<Peer> peer = _peerStorage->getUnusedPeer();
+        connectToPeer(peer);
       }
       if(_btRuntime->getConnections() == 0 &&
-	 !_pieceStorage->downloadFinished()) {
-	_btAnnounce->overrideMinInterval(BtAnnounce::DEFAULT_ANNOUNCE_INTERVAL);
+         !_pieceStorage->downloadFinished()) {
+        _btAnnounce->overrideMinInterval(BtAnnounce::DEFAULT_ANNOUNCE_INTERVAL);
       }
     }
   }
@@ -122,7 +129,7 @@ bool ActivePeerConnectionCommand::execute() {
   return false;
 }
 
-void ActivePeerConnectionCommand::connectToPeer(const PeerHandle& peer)
+void ActivePeerConnectionCommand::connectToPeer(const SharedHandle<Peer>& peer)
 {
   if(peer.isNull()) {
     return;
@@ -130,12 +137,14 @@ void ActivePeerConnectionCommand::connectToPeer(const PeerHandle& peer)
   peer->usedBy(e->newCUID());
   PeerInitiateConnectionCommand* command =
     new PeerInitiateConnectionCommand(peer->usedBy(), _requestGroup, peer, e,
-				      _btContext, _btRuntime);
+                                      _btRuntime);
   command->setPeerStorage(_peerStorage);
   command->setPieceStorage(_pieceStorage);
   e->commands.push_back(command);
-  logger->info(MSG_CONNECTING_TO_PEER,
-	       cuid, peer->ipaddr.c_str());
+  if(logger->info()) {
+    logger->info(MSG_CONNECTING_TO_PEER,
+                 util::itos(cuid).c_str(), peer->ipaddr.c_str());
+  }
 }
 
 void ActivePeerConnectionCommand::setBtRuntime
