@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -40,6 +40,7 @@
 #include "Logger.h"
 #include "LogFactory.h"
 #include "SimpleRandomizer.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
@@ -51,13 +52,13 @@ BtSeederStateChoke::BtSeederStateChoke():
 BtSeederStateChoke::~BtSeederStateChoke() {}
 
 BtSeederStateChoke::PeerEntry::PeerEntry
-(const SharedHandle<Peer>& peer, const struct timeval& now):
+(const SharedHandle<Peer>& peer):
   _peer(peer),
   _outstandingUpload(peer->countOutstandingUpload()),
   _lastAmUnchoking(peer->getLastAmUnchoking()),
-  _recentUnchoking(!_lastAmUnchoking.elapsed(TIME_FRAME)),
-  _uploadSpeed(peer->calculateUploadSpeed(now))
-  {}
+  _recentUnchoking(_lastAmUnchoking.difference(global::wallclock) < TIME_FRAME),
+  _uploadSpeed(peer->calculateUploadSpeed())
+{}
 
 bool
 BtSeederStateChoke::PeerEntry::operator<(const PeerEntry& rhs) const
@@ -68,7 +69,7 @@ BtSeederStateChoke::PeerEntry::operator<(const PeerEntry& rhs) const
     return false;
   }
   if(this->_recentUnchoking &&
-     this->_lastAmUnchoking.isNewer(rhs._lastAmUnchoking)) {
+     (this->_lastAmUnchoking > rhs._lastAmUnchoking)) {
     return true;
   } else if(rhs._recentUnchoking) {
     return false;
@@ -83,25 +84,26 @@ void BtSeederStateChoke::PeerEntry::disableOptUnchoking()
 }
 
 void BtSeederStateChoke::unchoke
-(std::deque<BtSeederStateChoke::PeerEntry>& peers)
+(std::vector<BtSeederStateChoke::PeerEntry>& peers)
 {
   int count = (_round == 2) ? 4 : 3;
 
   std::sort(peers.begin(), peers.end());
 
-  std::deque<PeerEntry>::iterator r = peers.begin();
-  for(; r != peers.end() && count; ++r, --count) {
+  std::vector<PeerEntry>::iterator r = peers.begin();
+  for(std::vector<PeerEntry>::iterator eoi = peers.end();
+      r != eoi && count; ++r, --count) {
     (*r).getPeer()->chokingRequired(false);
     _logger->info("RU: %s, ulspd=%u", (*r).getPeer()->ipaddr.c_str(),
-		  (*r).getUploadSpeed());
+                  (*r).getUploadSpeed());
   }
 
   if(_round < 2) {
     std::for_each(peers.begin(), peers.end(),
-		  std::mem_fun_ref(&PeerEntry::disableOptUnchoking));
+                  std::mem_fun_ref(&PeerEntry::disableOptUnchoking));
     if(r != peers.end()) {
       std::random_shuffle(r, peers.end(),
-			  *(SimpleRandomizer::getInstance().get()));
+                          *(SimpleRandomizer::getInstance().get()));
       (*r).getPeer()->optUnchoking(true);
       _logger->info("POU: %s", (*r).getPeer()->ipaddr.c_str());
     }
@@ -117,17 +119,10 @@ public:
 };
 
 class GenPeerEntry {
-private:
-  struct timeval _now;
 public:
-  GenPeerEntry()
-  {
-    gettimeofday(&_now, 0);
-  }
-
   BtSeederStateChoke::PeerEntry operator()(const SharedHandle<Peer>& peer) const
   {
-    return BtSeederStateChoke::PeerEntry(peer, _now);
+    return BtSeederStateChoke::PeerEntry(peer);
   }
 };
 
@@ -140,21 +135,22 @@ public:
 };
 
 void
-BtSeederStateChoke::executeChoke(const std::deque<SharedHandle<Peer> >& peerSet)
+BtSeederStateChoke::executeChoke
+(const std::vector<SharedHandle<Peer> >& peerSet)
 {
   _logger->info("Seeder state, %d choke round started", _round);
-  _lastRound.reset();
+  _lastRound = global::wallclock;
 
-  std::deque<PeerEntry> peerEntries;
+  std::vector<PeerEntry> peerEntries;
 
   std::for_each(peerSet.begin(), peerSet.end(), ChokingRequired());
 
   std::transform(peerSet.begin(), peerSet.end(),
-		 std::back_inserter(peerEntries), GenPeerEntry());
-	      
+                 std::back_inserter(peerEntries), GenPeerEntry());
+              
   peerEntries.erase(std::remove_if(peerEntries.begin(), peerEntries.end(),
-				   NotInterestedPeer()),
-		    peerEntries.end());
+                                   NotInterestedPeer()),
+                    peerEntries.end());
 
   unchoke(peerEntries);
   
