@@ -54,19 +54,20 @@ class PieceSelector;
 
 class HaveEntry {
 private:
-  int32_t cuid;
+  cuid_t cuid;
   size_t index;
-  Time registeredTime;
+  Timer registeredTime;
 public:
-  HaveEntry(int32_t cuid, size_t index):
+  HaveEntry(cuid_t cuid, size_t index, const Timer& registeredTime):
     cuid(cuid),
-    index(index) {}
+    index(index),
+    registeredTime(registeredTime) {}
 
-  int32_t getCuid() const { return cuid; }
+  cuid_t getCuid() const { return cuid; }
 
   size_t getIndex() const { return index; }
 
-  const Time& getRegisteredTime() const { return registeredTime; }
+  const Timer& getRegisteredTime() const { return registeredTime; }
 };
 
 class DefaultPieceStorage : public PieceStorage {
@@ -191,11 +192,11 @@ public:
 
   virtual size_t getPieceLength(size_t index);
 
-  virtual void advertisePiece(int32_t cuid, size_t index);
+  virtual void advertisePiece(cuid_t cuid, size_t index);
 
   virtual void
   getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
-                            int32_t myCuid, const Time& lastCheckTime);
+                            cuid_t myCuid, const Timer& lastCheckTime);
 
   virtual void removeAdvertisedPiece(time_t elapsed);
 
