@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -58,6 +58,7 @@
 #include "StringFormat.h"
 #include "Decoder.h"
 #include "RequestGroupMan.h"
+#include "wallclock.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigestHelper.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -66,12 +67,12 @@ namespace aria2 {
 
 static const size_t BUFSIZE = 16*1024;
 
-DownloadCommand::DownloadCommand(int cuid,
-				 const RequestHandle& req,
-				 const SharedHandle<FileEntry>& fileEntry,
-				 RequestGroup* requestGroup,
-				 DownloadEngine* e,
-				 const SocketHandle& s):
+DownloadCommand::DownloadCommand(cuid_t cuid,
+                                 const SharedHandle<Request>& req,
+                                 const SharedHandle<FileEntry>& fileEntry,
+                                 RequestGroup* requestGroup,
+                                 DownloadEngine* e,
+                                 const SocketHandle& s):
   AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
   _buf(new unsigned char[BUFSIZE])
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -83,11 +84,11 @@ DownloadCommand::DownloadCommand(int cuid,
     if(getOption()->getAsBool(PREF_REALTIME_CHUNK_CHECKSUM)) {
       std::string algo = _requestGroup->getDownloadContext()->getPieceHashAlgo();
       if(MessageDigestContext::supports(algo)) {
-	_messageDigestContext.reset(new MessageDigestContext());
-	_messageDigestContext->trySetAlgo(algo);
-	_messageDigestContext->digestInit();
-	
-	_pieceHashValidationEnabled = true;
+        _messageDigestContext.reset(new MessageDigestContext());
+        _messageDigestContext->trySetAlgo(algo);
+        _messageDigestContext->digestInit();
+        
+        _pieceHashValidationEnabled = true;
       }
     }
   }
@@ -112,14 +113,14 @@ bool DownloadCommand::executeInternal() {
     return false;
   }
   setReadCheckSocket(socket);
-  SegmentHandle segment = _segments.front();
+  SharedHandle<Segment> segment = _segments.front();
 
   size_t bufSize;
   if(segment->getLength() > 0) {
     if(static_cast<uint64_t>(segment->getPosition()+segment->getLength()) <=
        static_cast<uint64_t>(_fileEntry->getLastOffset())) {
       bufSize = std::min(segment->getLength()-segment->getWrittenLength(),
-			 BUFSIZE);
+                         BUFSIZE);
     } else {
       bufSize = std::min(static_cast<size_t>(_fileEntry->getLastOffset()-segment->getPositionToWrite()), BUFSIZE);
     }
@@ -147,12 +148,12 @@ bool DownloadCommand::executeInternal() {
 
   if(_contentEncodingDecoder.isNull()) {
     diskAdaptor->writeData(bufFinal, bufSizeFinal,
-			   segment->getPositionToWrite());
+                           segment->getPositionToWrite());
   } else {
     std::string out = _contentEncodingDecoder->decode(bufFinal, bufSizeFinal);
     diskAdaptor->writeData(reinterpret_cast<const unsigned char*>(out.data()),
-			   out.size(),
-			   segment->getPositionToWrite());
+                           out.size(),
+                           segment->getPositionToWrite());
     bufSizeFinal = out.size();
   }
 
@@ -178,16 +179,18 @@ bool DownloadCommand::executeInternal() {
        segment->getPositionToWrite() == _fileEntry->getLastOffset()) {
       segmentPartComplete = true;
     } else if(segment->getLength() == 0 && bufSize == 0 &&
-	      !socket->wantRead() && !socket->wantWrite()) {
+              !socket->wantRead() && !socket->wantWrite()) {
       segmentPartComplete = true;
     }
   } else if(!_transferEncodingDecoder.isNull() &&
-	    (segment->complete() || segment->getPositionToWrite() == _fileEntry->getLastOffset())) {
+            (segment->complete() || segment->getPositionToWrite() == _fileEntry->getLastOffset())) {
+    // In this case, transferEncodingDecoder is used and
+    // Content-Length is known.
     segmentPartComplete = true;
   } else if((_transferEncodingDecoder.isNull() ||
-	     _transferEncodingDecoder->finished()) &&
-	    (_contentEncodingDecoder.isNull() ||
-	     _contentEncodingDecoder->finished())) {
+             _transferEncodingDecoder->finished()) &&
+            (_contentEncodingDecoder.isNull() ||
+             _contentEncodingDecoder->finished())) {
     segmentPartComplete = true;
   }
 
@@ -201,36 +204,40 @@ bool DownloadCommand::executeInternal() {
       // If segment->getLength() == 0, the server doesn't provide
       // content length, but the client detected that download
       // completed.
-      logger->info(MSG_SEGMENT_DOWNLOAD_COMPLETED, cuid);
+      if(logger->info()) {
+        logger->info(MSG_SEGMENT_DOWNLOAD_COMPLETED, util::itos(cuid).c_str());
+      }
 #ifdef ENABLE_MESSAGE_DIGEST
 
       {
-	std::string expectedPieceHash =
-	  _requestGroup->getDownloadContext()->getPieceHash(segment->getIndex());
-	if(_pieceHashValidationEnabled && !expectedPieceHash.empty()) {
-	  if(segment->isHashCalculated()) {
-	    logger->debug("Hash is available! index=%lu",
-			  static_cast<unsigned long>(segment->getIndex()));
-	    validatePieceHash(segment, expectedPieceHash, segment->getHashString());
-	  } else {
-	    _messageDigestContext->digestReset();
-	    validatePieceHash(segment, expectedPieceHash,
-			      MessageDigestHelper::digest
-			      (_messageDigestContext.get(),
-			       _requestGroup->getPieceStorage()->getDiskAdaptor(),
-			       segment->getPosition(),
-			       segment->getLength()));
-	  }
-	} else {
-	  _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
-	}
+        std::string expectedPieceHash =
+          _requestGroup->getDownloadContext()->getPieceHash(segment->getIndex());
+        if(_pieceHashValidationEnabled && !expectedPieceHash.empty()) {
+          if(segment->isHashCalculated()) {
+            if(logger->debug()) {
+              logger->debug("Hash is available! index=%lu",
+                            static_cast<unsigned long>(segment->getIndex()));
+            }
+            validatePieceHash(segment, expectedPieceHash, segment->getHashString());
+          } else {
+            _messageDigestContext->digestReset();
+            validatePieceHash(segment, expectedPieceHash,
+                              MessageDigestHelper::digest
+                              (_messageDigestContext.get(),
+                               _requestGroup->getPieceStorage()->getDiskAdaptor(),
+                               segment->getPosition(),
+                               segment->getLength()));
+          }
+        } else {
+          _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
+        }
       }
 
 #else // !ENABLE_MESSAGE_DIGEST
       _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
 #endif // !ENABLE_MESSAGE_DIGEST
     } else {
-      // If segment is not cacnel here, in the next pipelining
+      // If segment is not canceled here, in the next pipelining
       // request, aria2 requests bad range
       // [FileEntry->getLastOffset(), FileEntry->getLastOffset())
       _requestGroup->getSegmentMan()->cancelSegment(cuid, segment);
@@ -249,58 +256,61 @@ bool DownloadCommand::executeInternal() {
 void DownloadCommand::checkLowestDownloadSpeed() const
 {
   // calculate downloading speed
-  if(peerStat->getDownloadStartTime().elapsed(startupIdleTime)) {
+  if(peerStat->getDownloadStartTime().difference(global::wallclock) >=
+     startupIdleTime) {
     unsigned int nowSpeed = peerStat->calculateDownloadSpeed();
     if(lowestDownloadSpeedLimit > 0 &&  nowSpeed <= lowestDownloadSpeedLimit) {
       throw DL_ABORT_EX2(StringFormat(EX_TOO_SLOW_DOWNLOAD_SPEED,
-				      nowSpeed,
-				      lowestDownloadSpeedLimit,
-				      req->getHost().c_str()).str(),
-			 downloadresultcode::TOO_SLOW_DOWNLOAD_SPEED);
+                                      nowSpeed,
+                                      lowestDownloadSpeedLimit,
+                                      req->getHost().c_str()).str(),
+                         downloadresultcode::TOO_SLOW_DOWNLOAD_SPEED);
     }
   }
 }
 
 bool DownloadCommand::prepareForNextSegment() {
   if(_requestGroup->downloadFinished()) {
+    // Remove in-flight request here.
+    _fileEntry->poolRequest(req);
     // If this is a single file download, and file size becomes known
     // just after downloading, set total length to FileEntry object
     // here.
     if(_requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
       const SharedHandle<FileEntry>& fileEntry =
-	_requestGroup->getDownloadContext()->getFirstFileEntry();
+        _requestGroup->getDownloadContext()->getFirstFileEntry();
       if(fileEntry->getLength() == 0) {
-	fileEntry->setLength
-	  (_requestGroup->getPieceStorage()->getCompletedLength());
+        fileEntry->setLength
+          (_requestGroup->getPieceStorage()->getCompletedLength());
       }
     }
 #ifdef ENABLE_MESSAGE_DIGEST
-    CheckIntegrityEntryHandle entry(new ChecksumCheckIntegrityEntry(_requestGroup));
+    SharedHandle<CheckIntegrityEntry> entry
+      (new ChecksumCheckIntegrityEntry(_requestGroup));
     if(entry->isValidationReady()) {
       entry->initValidator();
       // TODO do we need cuttrailinggarbage here?
       e->_checkIntegrityMan->pushEntry(entry);
     }
-    e->setNoWait(true);
-    e->setRefreshInterval(0);
 #endif // ENABLE_MESSAGE_DIGEST
     return true;
   } else {
     // The number of segments should be 1 in order to pass through the next
     // segment.
     if(_segments.size() == 1) {
-      SegmentHandle tempSegment = _segments.front();
+      SharedHandle<Segment> tempSegment = _segments.front();
       if(!tempSegment->complete()) {
-	return prepareForRetry(0);
+        return prepareForRetry(0);
       }
-      SegmentHandle nextSegment =
-	_requestGroup->getSegmentMan()->getSegment(cuid,
-						   tempSegment->getIndex()+1);
-      if(!nextSegment.isNull() && nextSegment->getWrittenLength() == 0) {
-	e->commands.push_back(this);
-	return false;
+      SharedHandle<SegmentMan> segmentMan = _requestGroup->getSegmentMan();
+      SharedHandle<Segment> nextSegment =
+        segmentMan->getCleanSegmentIfOwnerIsIdle
+        (cuid, tempSegment->getIndex()+1);
+      if(nextSegment.isNull()) {
+        return prepareForRetry(0);
       } else {
-	return prepareForRetry(0);
+        e->commands.push_back(this);
+        return false;
       }
     } else {
       return prepareForRetry(0);
@@ -311,18 +321,18 @@ bool DownloadCommand::prepareForNextSegment() {
 #ifdef ENABLE_MESSAGE_DIGEST
 
 void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
-					const std::string& expectedPieceHash,
-					const std::string& actualPieceHash)
+                                        const std::string& expectedPieceHash,
+                                        const std::string& actualPieceHash)
 {
   if(actualPieceHash == expectedPieceHash) {
     logger->info(MSG_GOOD_CHUNK_CHECKSUM, actualPieceHash.c_str());
     _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
   } else {
     logger->info(EX_INVALID_CHUNK_CHECKSUM,
-		 segment->getIndex(),
-		 util::itos(segment->getPosition(), true).c_str(),
-		 expectedPieceHash.c_str(),
-		 actualPieceHash.c_str());
+                 segment->getIndex(),
+                 util::itos(segment->getPosition(), true).c_str(),
+                 expectedPieceHash.c_str(),
+                 actualPieceHash.c_str());
     segment->clear();
     _requestGroup->getSegmentMan()->cancelSegment(cuid);
     throw DL_RETRY_EX
