@@ -45,11 +45,12 @@
 #include "DownloadContext.h"
 #include "a2functional.h"
 #include "RecoverableException.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
 FileAllocationCommand::FileAllocationCommand
-(int cuid, RequestGroup* requestGroup, DownloadEngine* e,
+(cuid_t cuid, RequestGroup* requestGroup, DownloadEngine* e,
  const SharedHandle<FileAllocationEntry>& fileAllocationEntry):
   RealtimeCommand(cuid, requestGroup, e),
   _fileAllocationEntry(fileAllocationEntry) {}
@@ -58,14 +59,15 @@ FileAllocationCommand::~FileAllocationCommand() {}
 
 bool FileAllocationCommand::executeInternal()
 {
-  if(_e->isHaltRequested()) {
+  if(_requestGroup->isHaltRequested()) {
+    _e->_fileAllocationMan->dropPickedEntry();
     return true;
   }
   _fileAllocationEntry->allocateChunk();
   if(_fileAllocationEntry->finished()) {
     if(logger->debug()) {
       logger->debug(MSG_ALLOCATION_COMPLETED,
-                    _timer.difference(),
+                    _timer.difference(global::wallclock),
                     util::itos(_requestGroup->getTotalLength(), true).c_str());
     }
     _e->_fileAllocationMan->dropPickedEntry();
@@ -89,8 +91,8 @@ bool FileAllocationCommand::executeInternal()
 bool FileAllocationCommand::handleException(Exception& e)
 {
   _e->_fileAllocationMan->dropPickedEntry();
-  logger->error(MSG_FILE_ALLOCATION_FAILURE, e, cuid);
-  logger->error(MSG_DOWNLOAD_NOT_COMPLETE, cuid,
+  logger->error(MSG_FILE_ALLOCATION_FAILURE, e, util::itos(cuid).c_str());
+  logger->error(MSG_DOWNLOAD_NOT_COMPLETE, util::itos(cuid).c_str(),
                 _requestGroup->getDownloadContext()->getBasePath().c_str());
   return true;
 }
