@@ -40,20 +40,21 @@
 #include "util.h"
 #include "URISelector.h"
 #include "LogFactory.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
 FileEntry::FileEntry(const std::string& path,
                      uint64_t length,
                      off_t offset,
-                     const std::deque<std::string>& uris):
-  path(path), _uris(uris), length(length), offset(offset),
-  extracted(false), requested(true),
+                     const std::vector<std::string>& uris):
+  path(path), _uris(uris.begin(), uris.end()), length(length), offset(offset),
+  requested(true),
   _singleHostMultiConnection(true),
   _logger(LogFactory::getInstance()) {}
 
 FileEntry::FileEntry():
-  length(0), offset(0), extracted(false), requested(false),
+  length(0), offset(0), requested(false),
   _singleHostMultiConnection(true),
   _logger(LogFactory::getInstance()) {}
 
@@ -70,7 +71,6 @@ FileEntry& FileEntry::operator=(const FileEntry& entry)
     path = entry.path;
     length = entry.length;
     offset = entry.offset;
-    extracted = entry.extracted;
     requested = entry.requested;
   }
   return *this;
@@ -92,7 +92,7 @@ off_t FileEntry::gtoloff(off_t goff) const
   return goff-offset;
 }
 
-void FileEntry::getUris(std::deque<std::string>& uris) const
+void FileEntry::getUris(std::vector<std::string>& uris) const
 {
   uris.insert(uris.end(), _spentUris.begin(), _spentUris.end());
   uris.insert(uris.end(), _uris.begin(), _uris.end());
@@ -125,14 +125,14 @@ FileEntry::getRequest
 {
   SharedHandle<Request> req;
   if(_requestPool.empty()) {
-    std::deque<std::string> pending;
+    std::vector<std::string> pending;
     while(1) {
       std::string uri = selector->select(this);
       if(uri.empty()) {
         return req;
       }
       req.reset(new Request());
-      if(req->setUrl(uri)) {
+      if(req->setUri(uri)) {
         if(!_singleHostMultiConnection) {
           if(inFlightHost(_inFlightRequests.begin(), _inFlightRequests.end(),
                           req->getHost())) {
@@ -173,7 +173,8 @@ FileEntry::findFasterRequest(const SharedHandle<Request>& base)
   // TODO hard coded value. See PREF_STARTUP_IDLE_TIME
   const int startupIdleTime = 10;
   if(basestat.isNull() ||
-     (basestat->getDownloadStartTime().elapsed(startupIdleTime) &&
+     (basestat->getDownloadStartTime().
+      difference(global::wallclock) >= startupIdleTime &&
       fastest->getAvgDownloadSpeed()*0.8 > basestat->calculateDownloadSpeed())){
     // TODO we should consider that "fastest" is very slow.
     SharedHandle<Request> fastestRequest = _requestPool.front();
@@ -217,13 +218,16 @@ void FileEntry::storePool(const SharedHandle<Request>& request)
 void FileEntry::poolRequest(const SharedHandle<Request>& request)
 {
   removeRequest(request);
-  storePool(request);
+  if(!request->removalRequested()) {
+    storePool(request);
+  }
 }
 
 bool FileEntry::removeRequest(const SharedHandle<Request>& request)
 {
   for(std::deque<SharedHandle<Request> >::iterator i =
-        _inFlightRequests.begin(); i != _inFlightRequests.end(); ++i) {
+        _inFlightRequests.begin(), eoi = _inFlightRequests.end();
+      i != eoi; ++i) {
     if((*i).get() == request.get()) {
       _inFlightRequests.erase(i);
       return true;
@@ -236,9 +240,10 @@ void FileEntry::removeURIWhoseHostnameIs(const std::string& hostname)
 {
   std::deque<std::string> newURIs;
   Request req;
-  for(std::deque<std::string>::const_iterator itr = _uris.begin(); itr != _uris.end(); ++itr) {
+  for(std::deque<std::string>::const_iterator itr = _uris.begin(),
+        eoi = _uris.end(); itr != eoi; ++itr) {
     if(((*itr).find(hostname) == std::string::npos) ||
-       (req.setUrl(*itr) && (req.getHost() != hostname))) {
+       (req.setUri(*itr) && (req.getHost() != hostname))) {
       newURIs.push_back(*itr);
     }
   }
@@ -287,20 +292,24 @@ void FileEntry::reuseUri(size_t num)
   std::sort(uris.begin(), uris.end());
   uris.erase(std::unique(uris.begin(), uris.end()), uris.end());
 
-  std::deque<std::string> errorUris(_uriResults.size());
+  std::vector<std::string> errorUris(_uriResults.size());
   std::transform(_uriResults.begin(), _uriResults.end(),
                  errorUris.begin(), std::mem_fun_ref(&URIResult::getURI));
   std::sort(errorUris.begin(), errorUris.end());
   errorUris.erase(std::unique(errorUris.begin(), errorUris.end()),
                   errorUris.end());
      
-  std::deque<std::string> reusableURIs;
+  std::vector<std::string> reusableURIs;
   std::set_difference(uris.begin(), uris.end(),
                       errorUris.begin(), errorUris.end(),
                       std::back_inserter(reusableURIs));
   size_t ininum = reusableURIs.size();
   if(_logger->debug()) {
     _logger->debug("Found %u reusable URIs", static_cast<unsigned int>(ininum));
+    for(std::vector<std::string>::const_iterator i = reusableURIs.begin(),
+          eoi = reusableURIs.end(); i != eoi; ++i) {
+      _logger->debug("URI=%s", (*i).c_str());
+    }
   }
   // Reuse at least num URIs here to avoid to
   // run this process repeatedly.
@@ -324,7 +333,51 @@ void FileEntry::releaseRuntimeResource()
 {
   _requestPool.clear();
   _inFlightRequests.clear();
-  _uriResults.clear();
+}
+
+template<typename InputIterator, typename T>
+static InputIterator findRequestByUri
+(InputIterator first, InputIterator last, const T& uri)
+{
+  for(; first != last; ++first) {
+    if(!(*first)->removalRequested() && (*first)->getUri() == uri) {
+      return first;
+    }
+  }
+  return last;
+}
+
+bool FileEntry::removeUri(const std::string& uri)
+{
+  std::deque<std::string>::iterator itr =
+    std::find(_spentUris.begin(), _spentUris.end(), uri);
+  if(itr == _spentUris.end()) {
+    itr = std::find(_uris.begin(), _uris.end(), uri);
+    if(itr == _uris.end()) {
+      return false;
+    } else {
+      _uris.erase(itr);
+      return true;
+    }
+  } else {
+    _spentUris.erase(itr);
+    SharedHandle<Request> req;
+    std::deque<SharedHandle<Request> >::iterator riter =
+      findRequestByUri(_inFlightRequests.begin(), _inFlightRequests.end(), uri);
+    if(riter == _inFlightRequests.end()) {
+      riter = findRequestByUri(_requestPool.begin(), _requestPool.end(), uri);
+      if(riter == _requestPool.end()) {
+        return true;
+      } else {
+        req = *riter;
+        _requestPool.erase(riter);
+      }
+    } else {
+      req = *riter;
+    }
+    req->requestRemoval();
+    return true;
+  }
 }
 
 } // namespace aria2
