@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -36,17 +36,15 @@
 #define _D_REQUEST_SLOT_H_
 
 #include "common.h"
-
-#include <deque>
-
-#include "TimeA2.h"
+#include "TimerA2.h"
 #include "Piece.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
 class RequestSlot {
 private:
-  Time dispatchedTime;
+  Timer dispatchedTime;
   size_t index;
   uint32_t begin;
   size_t length;
@@ -62,26 +60,31 @@ private:
   // inlined for performance reason
   void copy(const RequestSlot& requestSlot)
   {
+    dispatchedTime = requestSlot.dispatchedTime;
     index = requestSlot.index;
     begin = requestSlot.begin;
     length = requestSlot.length;
     blockIndex = requestSlot.blockIndex;
-    dispatchedTime = requestSlot.dispatchedTime;
     _piece = requestSlot._piece;
   }
 public:
   
   RequestSlot(size_t index, uint32_t begin, size_t length, size_t blockIndex,
-	      const SharedHandle<Piece>& piece = SharedHandle<Piece>()):
+              const SharedHandle<Piece>& piece = SharedHandle<Piece>()):
+    dispatchedTime(global::wallclock),
     index(index), begin(begin), length(length), blockIndex(blockIndex),
     _piece(piece) {}
 
-  RequestSlot(const RequestSlot& requestSlot)
-  {
-    copy(requestSlot);
-  }
+  RequestSlot(const RequestSlot& requestSlot):
+    dispatchedTime(requestSlot.dispatchedTime),
+    index(requestSlot.index),
+    begin(requestSlot.begin),
+    length(requestSlot.length),
+    blockIndex(requestSlot.blockIndex),
+    _piece(requestSlot._piece) {}
 
-  RequestSlot():index(0), begin(0), length(0), blockIndex(0) {}
+  RequestSlot():dispatchedTime(0), index(0), begin(0), length(0), blockIndex(0)
+  {}
 
   ~RequestSlot() {}
 
@@ -113,11 +116,9 @@ public:
     }
   }
 
-  void setDispatchedTime();
   void setDispatchedTime(time_t secFromEpoch);
 
-  bool isTimeout(const struct timeval& now, time_t timeoutSec) const;
-  unsigned int getLatencyInMillis() const;
+  bool isTimeout(time_t timeoutSec) const;
 
   size_t getIndex() const { return index; }
   void setIndex(size_t index) { this->index = index; }
@@ -141,8 +142,6 @@ public:
   static bool isNull(const RequestSlot& requestSlot);
 };
 
-typedef std::deque<RequestSlot> RequestSlots;
-
 } // namespace aria2
 
 #endif // _D_REQUEST_SLOT_H_
